/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "TelescopeCMD.h"

#if TELESCOPE(COMMAND_SYSTEM)
namespace Telescope
{
	TelescopeCMD::TelescopeCMD()
	{
		RegisterAllCommands();
	}


#if TELESCOPE(MEMORY_TRACING)
	void TelescopeCMD::MemoryTracing_EnableMemoryLeakBacktrace( const char* pParam )
	{
		TelescopeMemoryTracing::GetInstance()->EnableMemoryLeakBacktrace();
	}


	void TelescopeCMD::MemoryTracing_DisableMemoryLeakBacktrace( const char* pParam )
	{
		TelescopeMemoryTracing::GetInstance()->DisableMemoryLeakBacktrace();
	}


	void TelescopeCMD::MemoryTracing_DumpMemoryLeakBacktrace( const char* pParam )
	{
		TelescopeMemoryTracing::GetInstance()->DumpMemoryLeakBacktrace( true, 1 );
	}


	void TelescopeCMD::MemoryTracing_ClearMemoryLeakBacktrace( const char* pParam )
	{
		TelescopeMemoryTracing::GetInstance()->ClearMemoryLeakBacktrace();
	}
#endif


	void TelescopeCMD::ExecuteCMD( const char* pCMD, const char* pParam )
	{
		auto itr = g_mCommandCB.find( pCMD );
		if ( itr != g_mCommandCB.end() )
			( itr->second )( pParam );
	}


	void TelescopeCMD::RegisterCommand( const char* pCMD, std::function<void( const char* )> pFn )
	{
		g_mCommandCB[pCMD] = pFn;
	}


	void TelescopeCMD::RegisterAllCommands()
	{
#if TELESCOPE(MEMORY_TRACING)
		RegisterCommand( "MemoryTracing_EnableMemoryLeakBacktrace", std::bind( &TelescopeCMD::MemoryTracing_EnableMemoryLeakBacktrace, this, std::placeholders::_1 ) );
		RegisterCommand( "MemoryTracing_DisableMemoryLeakBacktrace", std::bind( &TelescopeCMD::MemoryTracing_DisableMemoryLeakBacktrace, this, std::placeholders::_1 ) );
		RegisterCommand( "MemoryTracing_DumpMemoryLeakBacktrace", std::bind( &TelescopeCMD::MemoryTracing_DumpMemoryLeakBacktrace, this, std::placeholders::_1 ) );
		RegisterCommand( "MemoryTracing_ClearMemoryLeakBacktrace", std::bind( &TelescopeCMD::MemoryTracing_ClearMemoryLeakBacktrace, this, std::placeholders::_1 ) );
#endif
	}
};
#endif
